# -*- coding: utf-8 -*-
"""Sendgrid Class file for Retreat App."""
# import sendgrid
from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import ClickTracking
from sendgrid.helpers.mail import Content
from sendgrid.helpers.mail import From
from sendgrid.helpers.mail import Mail
from sendgrid.helpers.mail import MimeType
from sendgrid.helpers.mail import Subject
from sendgrid.helpers.mail import To
from sendgrid.helpers.mail import TrackingSettings


class Sendgrid:
    """Sendgrid Class for Retreat App."""

    def __init__(self, api_key):
        """Initialize Sendgrid Class."""
        self.client = SendGridAPIClient(api_key=api_key)

    def send_email(self, to, subject, html_content):
        """Send an email to a user."""
        # create an email message with sendgrid
        message = Mail()
        message.from_email = From("noreply@broadinstitute.org")
        message.to = [To(email=to)]
        message.subject = Subject(subject)
        message.content = Content(MimeType.html, html_content)

        # disable click tracking
        tracking_settings = TrackingSettings()
        tracking_settings.click_tracking = ClickTracking(False, False)
        message.tracking_settings = tracking_settings

        # send email
        try:
            response = self.client.send(message)
        except Exception as sendgrid_error:
            print(f"ERROR: Failed sending Sendgrid email: {sendgrid_error}")
            return None
        return response



# -*- coding: utf-8 -*-
"""Retreat class file."""
import os
import requests

from google.cloud import secretmanager_v1


class Retreat:
    """Retreat class."""

    def __init__(self):
        """Initialize a new Retreat instance."""
        self.project = None

    def firestore(self):
        """Return a Retreat Firestore object."""
        from bits.retreat.firestore import Firestore
        return Firestore()

    def get_domains(self, registrants):
        """Return a dict or domains."""
        domains = {}
        for r in registrants:
            domain = r["domain"]
            if domain not in domains:
                domains[domain] = []
            domains[domain].append(r)
        return domains

    def get_project(self):
        """Return the project ID of the current environment."""
        if "GCP_PROJECT" in os.environ:
            return os.environ["GCP_PROJECT"]
        if "GOOGLE_CLOUD_PROJECT" in os.environ:
            return os.environ["GOOGLE_CLOUD_PROJECT"]
        return requests.get(
            "http://metadata.google.internal/computeMetadata/v1/project/project-id",
            headers={"Metadata-Flavor": "Google"},
            timeout=10,
        ).text

    def get_secret(self, name):
        """Return the auth data from the request_json."""
        if not self.project:
            self.project = self.get_project()
        client = secretmanager_v1.SecretManagerServiceClient()
        secret_path = client.secret_version_path(self.project, name, "latest")
        request = {"name": secret_path}
        return client.access_secret_version(request=request).payload.data.decode("utf-8")

    def swoogo(self):
        """Connect to Swoogo API."""
        from bits.retreat.swoogo import Swoogo
        params = {
            "api_key": os.environ.get("SWOOGO_API_KEY"),
            "api_secret": self.get_secret("swoogo-api-secret"),
        }
        return Swoogo(**params)



# -*- coding: utf-8 -*-
"""Helpers module for Get Swoogo Data cloud function."""
import logging


class Registrant:
    """Registrant class."""

    def __init__(self, registrant, config):
        """Initialize a Registrant class instance."""
        self.config = config
        self.registrant = registrant

        # initialize registrant
        self.approval = None
        self.emplid = None
        self.ippia_response = None
        self.person = None
        self.status = None

        # initialize registration
        self.registration = {
            # email doamain
            "domain": None,

            # workday info
            "broad_email": None,
            "emplid": None,
            "job_class": None,
            "terminated": None,
            "title": None,
            "username": None,
            "workday_home_institution": None,
            "worker_status": None,
            "worker_sub_type": None,
            "worker_type": None,

            # ippia info
            "ippia_response": None,

            # approval status
            "status": None,
        }

        # add config fields from registrant to registration dict
        self.get_config_fields()

        # set registration data
        self.id = str(self.registration["id"])
        self.email = self.registration["email"].lower()
        self.registration["domain"] = self.get_domain()

        # get judge and poster data
        # self.registration["judge_categories"] = self.get_judge_categories()
        self.registration["poster_authors"] = self.get_poster_authors()
        self.registration["poster_presenters"] = self.get_poster_presenters()

    def get_approval_data(self, approvals):
        """Add approval data."""
        if self.id in approvals:
            self.approval = approvals[self.id]

    def get_config_fields(self):
        """Populate registration dict with config fields from registrant."""
        # add basic fields
        for key in self.config["basic_fields"]:
            if key == "reg_type_id":
                self.registration["personal_statement"] = self.registrant[key]["value"]
            else:
                self.registration[key] = self.registrant[key]

        # add poster and registration fields
        for section in ["poster_fields", "registration_fields"]:
            for param in self.config[section]:
                data = self.config[section][param]
                key = data["key"]
                if data["type"] in ["textArea", "textInput", "checkboxList"]:
                    self.registration[param] = self.registrant[key]
                else:
                    self.registration[param] = self.registrant[key]["value"]

    def get_domain(self):
        """Return the domain of the registrant's email address."""
        hostname = self.email.split("@")[1]
        return ".".join(hostname.split(".")[-2:])

    def get_gaia_people_data(self, emails):
        """Add gaia people data for this registrant."""
        if not self.email or self.email not in emails:
            # print(f"Person not found: {self.email}")
            return
        self.person = emails[self.email]
        self.emplid = self.person["emplid"]

        # set people fields
        self.registration["broad_email"] = self.person["email_username"]
        self.registration["emplid"] = self.emplid
        self.registration["job_class"] = self.person["job_class"]
        self.registration["terminated"] = self.person["terminated"]
        self.registration["title"] = self.person["title"]
        self.registration["username"] = self.person["username"]
        self.registration["workday_home_institution"] = self.person["home_institution"]
        self.registration["worker_status"] = self.person["worker_status"]
        self.registration["worker_sub_type"] = self.person["worker_sub_type"]
        self.registration["worker_type"] = self.person["worker_type"]

    def get_ippia_data(self, emplids):
        """Add IPPIA data for this registrant."""
        if not self.emplid or self.emplid not in emplids:
            return
        self.ippia_response = emplids[self.emplid]["Response"]
        self.registration["ippia_response"] = self.ippia_response

    def get_judge_categories(self):
        """Return the judge categories for the registrant."""
        categories = []
        for cat in self.registration["judge_categories"]:
            name = cat["value"]
            if name:
                categories.append(name)
        return categories

    def get_poster_authors(self):
        """Return the poster authors for a registrant."""
        authors = []
        for line in self.registration["poster_authors"].split("\n"):
            for author in line.split(","):
                for name in author.split(" and "):
                    name = name.strip()
                    if name:
                        authors.append(name.strip())
        return authors

    def get_poster_presenters(self):
        """Return the poster presenters for a registrant."""
        presenters = []
        for line in self.registration["poster_presenters"].split("\n"):
            for presenter in line.split(","):
                for name in presenter.split(" and "):
                    name = name.strip()
                    if name:
                        presenters.append(name.strip())
        return presenters

    def get_status(self):
        """Return the approval status of the registrant."""
        if self.person:

            # pre-approve broadies in any of these job classes
            if self.person["job_class"] in [
                "Affiliate Member",
                "Associate Member",
                "Core Member",
                "Employee",
                "Institute Member",
                "Senior Associate Member",
            ] and not self.person["terminated"]:
                self.status = "Pre-Approved"

            # pre-approve paid affiliates
            elif (self.registration["personal_statement"] == "I am a Broad Employee."
                    and self.person["worker_type"] == "Employee"
                    and self.person["worker_sub_type"] == "Paid Affiliate (Fixed Term)"
                    and self.ippia_response == "Accepted"
                    and not self.person["terminated"]):
                self.status = "Pre-Approved"

            # alert about terminated folks
            elif self.person["terminated"] and "@broadinstitute.org" in self.email and self.registration["registration_status"] in ["attended", "confirmed"]:
                logging.error("Terminated Broad Person: {}".format(self.email))

            elif self.person["terminated"] and self.registration["registration_status"] in ["attended", "confirmed"]:
                logging.warning("Former Broad Person: {}".format(self.email))

        # overide status with saved approval status
        if self.approval:
            self.status = self.approval["status"]

        self.registration["status"] = self.status

    def to_dict(self):
        """Return a dict of the Registrant."""
        return self.registration



# -*- coding: utf-8 -*-
"""Firestore class for Retreat app."""
import datetime
from google.cloud import firestore


class Firestore:
    """Firestore class."""

    def __init__(self):
        """Initialize a new Firestore instance."""
        self.client = firestore.Client()

    #
    # Firestore approvals
    #
    def approve_approval(self, rid, user):
        """Approve a single approval in Firestore."""
        self.update_approval_status(rid, "Approved", user)

    def reject_approval(self, rid, user):
        """Reject a single approval in Firestore."""
        self.update_approval_status(rid, "Rejected", user)

    def update_approval_status(self, rid, status, user):
        """Reject a single approval in Firestore."""
        updated = datetime.datetime.now()
        data = {
            "registrant_id": int(rid),
            "updated_by": user.email,
            "status": status,
            "updated": updated,
        }
        self.client.collection("approvals").document(rid).set(data)

    #
    # Firestore notes
    #
    def get_note(self, rid):
        """Return a single note from firestore."""
        return self.client.collection("notes").document(rid).get().to_dict()

    def get_notes_dict(self):
        """Return a dict of notes."""
        notes = {}
        for doc in self.client.collection("notes").stream():
            notes[doc.id] = doc.to_dict()
        return notes

    def update_note(self, rid, note):
        """Update a single note in firestore."""
        if not note:
            return
        ref = self.client.collection("notes").document(rid)
        doc = ref.get()
        if doc.exists:
            data = doc.to_dict()
            history = data.get("history", [])
            if "note" in data:
                history.append(data["note"])
            data["history"] = history
            data["note"] = note
        else:
            data = {"id": rid, "note": note, "history": []}
        ref.set(data)

    #
    # Firestore registrants
    #
    def approve_registrant(self, rid):
        """Approve a single registrant in Firestore."""
        self.update_registrant_status(rid, "Approved")

    def get_registrant(self, rid):
        """Return a single registrant."""
        return self.client.collection("registrants").document(rid).get().to_dict()

    def get_registrants(self, domain=None, status=None, registration_status=None):
        """Return a list of registrants."""
        registrants = []
        query = self.client.collection("registrants")
        if domain:
            query = query.where("domain", "==", domain)
        if status:
            query = query.where("status", "==", status)
        if registration_status:
            query = query.where("registration_status", "==", registration_status)
        for doc in query.stream():
            registrants.append(doc.to_dict())
        return registrants

    def reject_registrant(self, rid):
        """Reject a single registrant in Firestore."""
        self.update_registrant_status(rid, "Rejected")

    def update_registrant_status(self, rid, status):
        """Update a single registrant in Firestore."""
        ref = self.client.collection("registrants").document(rid)
        ref.update({"status": status})



# -*- coding: utf-8 -*-
"""Hopin class file."""
import requests


class Hopin:
    """Hopin class."""

    def __init__(self, token):
        """Initialize a class instance."""
        self.token = token
        self.headers = {
            "Accept": "application/json",
            "Authorization": f"Bearer {self.token}",
            "Content-type": "application/json",
        }
        self.base_url = "https://api.hopin.com/v1"

        self.event_id = "uOWSaHxaRtjBBIsMLGQzMGxnK"
        self.organization_id = "moDz7NUY5qBaZnDP1QcgsUV5Q"
        self.ticket_id = "mVEJ1B4Y8RWcQ6nDATsMGecVT"

    def get_list(self, url, params={}):
        """Return a response from a paginated list."""
        response = requests.get(url, headers=self.headers, params=params).json()
        items = response.get("data", [])
        while response["links"].get("next"):
            url = response["links"]["next"]
            response = requests.get(url, headers=self.headers).json()
            items += response.get("data", [])
        return items

    # Events
    def get_registrations(self):
        """Return a list of registrations."""
        url = f"{self.base_url}/events/{self.event_id}/registrations"
        return self.get_list(url)

    def get_registrations_by_email(self):
        """Return a dict of registrations by Email."""
        registrations = {}
        for r in self.get_registrations():
            if r["attributes"]["refunded"]:
                continue
            email = r["attributes"]["email"]
            if email not in registrations:
                registrations[email] = r
            else:
                print(f"Duplicate registration email: {email}")
        return registrations

    # Organizations
    def get_organizations(self):
        """Return a list of organizations."""
        url = f"{self.base_url}/organizations"
        return self.get_list(url)

    def get_events(self):
        """Return a list of events for the organization."""
        url = f"{self.base_url}/organizations/{self.organization_id}/events"
        return self.get_list(url)

    # Tickets
    def create_magic_link(self, email, first_name, last_name):
        """Return a magic link for a user."""
        url = f"{self.base_url}/tickets/{self.ticket_id}/magicLinks"
        body = {
            "data": {
                "type": "magicLink",
                "attributes": {
                    "email": email,
                    # "extraFields": {
                    #     "property1": "string",
                    #     "property2": "string"
                    # },
                    "firstName": first_name,
                    # "headline": "string",
                    "lastName": last_name,
                    # "metadata": {
                    #     "property1": "string",
                    #     "property2": "string"
                    # },
                    # "registrationId": "string"
                }
            }
        }
        response = requests.post(url, json=body, headers=self.headers)
        response.raise_for_status()
        return response.json()

    def get_magic_links(self):
        """Return a list of magic links for the ticket."""
        url = f"{self.base_url}/tickets/{self.ticket_id}/magicLinks"
        return self.get_list(url)

    def get_magic_links_by_email(self):
        """Return a dict of magic links by email address."""
        magic_links = {}
        for ml in self.get_magic_links():
            email = ml["attributes"]["email"]
            if email not in magic_links:
                magic_links[email] = ml
            else:
                print(f"Duplicate magic link email: {email}")
        return magic_links

    def get_tickets(self):
        """Return a list of tickets."""
        url = f"{self.base_url}/events/{self.event_id}/tickets"
        return self.get_list(url)



# -*- coding: utf-8 -*-
"""Swoogo class file."""

import requests


class Swoogo(object):
    """Swoogo class."""

    def __init__(self, api_key, api_secret, verbose=False):
        """Initialize a Swoogo class instance."""
        self.api_key = api_key
        self.api_secret = api_secret
        self.verbose = verbose

        self.base_url = "https://www.swoogo.com/api/v1"

        self.access_token = self._get_access_token()

        self.headers = {
            "Authorization": f"Bearer {self.access_token}",
        }

    def _get_access_token(self):
        """Return a bearer token for making authorized requests."""
        headers = {
            "Content-Type": "application/x-www-form-urlencoded;charset=UTF-8",
        }
        url = f"{self.base_url}/oauth2/token.json"
        data = "grant_type=client_credentials"
        auth = (self.api_key, self.api_secret)
        response = requests.post(url, auth=auth, headers=headers, data=data, timeout=10)
        return response.json().get("access_token")

    def _get_list_items(self, url, headers, params):
        """Return the results from a paginated list."""
        response = requests.get(url, headers=headers, params=params, timeout=60)
        response_data = response.json()
        items = response_data.get("items", [])

        next_url = response_data.get("_links", {}).get("next", {}).get("href")
        while next_url:
            response = requests.get(next_url, headers=headers, params=params, timeout=60)
            response_data = response.json()
            items.extend(response_data.get("items", []))
            next_url = response_data.get("_links", {}).get("next", {}).get("href")

        return items

    #
    # Events
    #
    def get_event(self, event_id):
        """Return a single Swoogo event."""
        url = f"{self.base_url}/events/{event_id}.json"
        params = {}
        response = requests.get(url, headers=self.headers, params=params, timeout=30)
        return response.json()

    def get_events(self):
        """Get all events from swoogo."""
        url = f"{self.base_url}/events.json"
        params = {}
        return self._get_list_items(url, headers=self.headers, params=params)

    #
    # Questions
    #
    def get_question(self, question_id):
        """Get a question from swoogo."""
        url = f"{self.base_url}/event-questions/{question_id}.json"
        params = {}
        response = requests.get(url, headers=self.headers, params=params, timeout=30)
        return response.json()

    def get_questions(self, event_id):
        """Get all questions from an event in swoogo."""
        url = f"{self.base_url}/event-questions.json"
        params = {
            "event_id": event_id,
            "fields": "id,name,attribute,type,page_id,sort",
            "per-page": 200,
        }
        return self._get_list_items(url, headers=self.headers, params=params)

    #
    # Registrants
    #
    def approve_swoogo_registration(self, rid, approval_field):
        """Approve a single registration in Swoogo."""
        url = f"{self.base_url}/registrants/update/{rid}.json"
        payload = {approval_field: "TRUE"}
        response = requests.put(url, headers=self.headers, json=payload, timeout=30)
        if response.status_code == 200:
            return True
        return False

    def cancel_swoogo_registration(self, rid):
        """Cancel a single registration in Swoogo."""
        url = f"{self.base_url}/registrants/update/{rid}.json"
        payload = {"registration_status": "cancelled"}
        response = requests.put(url, headers=self.headers, json=payload, timeout=30)
        if response.status_code == 200:
            return True
        return False

    def get_registrants(self, event_id, fields=None):
        """Get all registrants from an event in swoogo."""
        url = f"{self.base_url}/registrants.json"
        if not fields:
            fields = [
                "id",
                "first_name",
                "last_name",
                "email",
                "registration_status",
                "reg_type_id",
            ]
        params = {
            "event_id": event_id,
            "fields": ",".join(fields),
            "per-page": 200,
        }
        return self._get_list_items(url, headers=self.headers, params=params)

    def get_registrants_by_domain(self, event_id=None, registrants=None):
        """Return a dict of registrants by their email domain name."""
        if not event_id and not registrants:
            raise ValueError("Must provide either event_id or registrants")
        if not registrants:
            registrants = self.get_registrants(event_id)
        domains = {}
        for r in registrants:
            domain = r["domain"]
            if domain not in domains:
                domains[domain] = []
            domains[domain].append(r)
        return domains

    def get_registrants_by_id(self, event_id=None, registrants=None):
        """Return a dict of registrants by their registration id."""
        if not event_id and not registrants:
            raise ValueError("Must provide either event_id or registrants")
        if not registrants:
            registrants = self.get_registrants(event_id)
        reg_ids = {}
        for r in registrants:
            rid = r["id"]
            reg_ids[rid] = r
        return reg_ids

    #
    # Get Registrant Types
    #
    def get_registrant_types(self, event_id):
        """Get all registrant types from an event in swoogo."""
        url = f"{self.base_url}/reg-types.json"
        params = {
            "event_id": event_id,
            "per-page": 200,
        }
        return self._get_list_items(url, headers=self.headers, params=params)

    def get_registrant_types_dict(self, event_id):
        """Return a dict of registrant types for the given event."""
        types = {}
        for reg_type in self.get_registrant_types(event_id):
            type_id = reg_type["id"]
            types[type_id] = reg_type
        return types



# -*- coding: utf-8 -*-
"""Pathable class file."""
import os
import requests

from bits.pathable import Pathable as Client

PROJECT = os.environ.get('GOOGLE_CLOUD_PROJECT')


class Pathable:
    """Pathable class."""

    def __init__(self, api_key, community_id="JwWkx4QL5sxFgy229"):
        """Initialize a new Pathable instance."""
        self.api_key = api_key
        self.client = Client(api_key)
        self.community_id = community_id

    def create_pathable_person(self, r):
        """Create a person in pathable."""
        email = r["email"]
        first_name = r["first_name"]
        last_name = r["last_name"]
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "X-API-KEY": self.api_key,
        }
        params = {}
        body = {
            "_email": email,
            "_profile": {
                "firstName": first_name,
                "lastName": last_name,
                "_templateId": "xWCj6xgHu6TXveybt",
            }
        }
        try:
            response = requests.post(
                "https://api.pathable.co/v1/people",
                json=body,
                headers=headers,
                params=params,
            )
            response.raise_for_status()
            print(response.json())
        except Exception as e:
            print(f"ERROR: Failed to create person: {email} [{e}]")

    def create_pathable_profile(self, r):
        """Create a profile in pathable."""
        email = r["email"]
        first_name = r["first_name"]
        last_name = r["last_name"]
        person_id = r["person_id"]
        p = Pathable(self.api_key)
        body = {
            "firstName": first_name,
            "lastName": last_name,
            "personId": person_id,
            "communityId": self.community_id,
            "_templateId": "xWCj6xgHu6TXveybt",
        }
        try:
            # print(f"Skipping creating profile: {email}...")
            print(p.create_profile(body))
        except Exception as e:
            print(f"ERROR: Failed to create profile: {email} [{e}]")

    def get_pathable_people(self):
        """Return a dict of pathable people by email."""
        p = Pathable(self.api_key)
        people = {}
        for person in p.get_people():
            if "email" not in person:
                continue
            email = person["email"]
            people[email] = person
        return people

    def get_pathable_profiles(self):
        """Return a dict of pathable profiles by email."""
        profiles = {}
        p = Pathable(self.api_key)
        for profile in p.get_profiles():
            if "email" not in profile:
                continue
            email = profile["email"]
            profiles[email] = profile
        return profiles

    def update_pathable_people(self, registrants):
        """Update the people in Pathable."""
        print(f"Registrants: {len(registrants)}")

        pathable_people = self.get_pathable_people()
        print(f"Pathable People: {len(pathable_people)}")

        # group the registrants by status
        approved = []
        rejected = []
        terms = []
        unapproved = []
        for email in registrants:
            r = registrants[email]
            emplid = r["emplid"]
            status = r["status"]
            terminated = r["terminated"]

            # skip terminated broadies
            if "@broadinstitute.org" in email and emplid and terminated:
                print(f"Terminated Broadie: {email}")
                terms.append(email)
                continue

            # skip rejected registrants
            if status in ["Rejected"]:
                rejected.append(email)
                continue

            # skip un-approved registrants
            if not status or status in ["Needs Update", "Updated"]:
                unapproved.append(email)
                continue

            # find approved folks
            if status in ["Approved", "Pre-Approved"]:
                approved.append(email)
            else:
                print(f"ERROR: Unknown status for {email}: {status}")
                import json
                print(json.dumps(r, indent=2, sort_keys=True))

        # get pathable people to add
        add_people = []
        add_profiles = []
        for email in approved:
            r = registrants[email]
            if email not in pathable_people:
                add_people.append(r)
            elif self.community_id not in [profile["communityId"] for profile in pathable_people[email]["profiles"]]:
                r["person_id"] = pathable_people[email]["_id"]
                add_profiles.append(r)

        # get pathable_people to remove
        pathable_staff = []
        projection_staff = []
        remove = []
        unregistered = []
        for email in pathable_people:
            p = pathable_people[email]
            if email in approved:
                continue
            if self.community_id not in [profile["communityId"] for profile in p["profiles"]]:
                continue
            if "@pathable.com" in email:
                pathable_staff.append(email)
            elif "@projection.com" in email:
                projection_staff.append(email)
            elif email not in registrants:
                unregistered.append(email)
            else:
                status = remove.append(email)

        # output people to add
        output = f"\nPeople to Add: {len(add_people)}\n"
        for r in sorted(add_people, key=lambda x: x["email"]):
            output += f"   + {r['email']}\n"
            self.create_pathable_person(r)

        # output profiles to add
        output += f"\nProfiles to Add: {len(add_profiles)}\n"
        for r in sorted(add_profiles, key=lambda x: x["email"]):
            output += f"   + {r['email']}\n"
            self.create_pathable_profile(r)

        # output people to remove
        output += f"\nRemove: {len(remove)}\n"
        for email in sorted(remove):
            note = "Unknown"
            if email in registrants:
                status = registrants[email]["status"]
                if status:
                    note = status
                else:
                    note = "Needs Approval"
            output += f"   - {email} ({note})\n"

        # output unregistered profiles
        output += f"\nUnregistered: {len(unregistered)}\n"
        for email in sorted(unregistered):
            output += f"   - {email}\n"

        # output pathable staff
        output += f"\nPathable Staff: {len(pathable_staff)}\n"
        for email in sorted(pathable_staff):
            output += f"   * {email}\n"

        # output projection staff
        output += f"\nProjection Staff: {len(projection_staff)}\n"
        for email in sorted(projection_staff):
            output += f"   * {email}\n"

        print(output)
        return output
